@@ -1,6 +1,6 @@
 #' Package rdbnomics
 #'
-#' DBnomics R client.
+#' DBnomics R client (<https://db.nomics.world/>).
 #'
 #' @docType package
 #' @name rdbnomics
@@ -12,7 +12,9 @@ NULL
 if (getRversion() >= "2.15.1") {
   vars <- c(
     ".", ":=", "value", "dotI", "period", "period_start_day", "series_code",
-    "filtered", "original_period", "series_name", "original_value"
+    "filtered", "original_period", "series_name", "original_value",
+    "period_middle_day", "code", "name", "A", "B", "C", "k", "V1", "V2",
+    "dataset_code"
   )
   utils::globalVariables(unique(vars))
 }
\ No newline at end of file
