@@ -168,10 +168,13 @@ deploy <- function(DT, columns = NULL, reference_column = "value") {
           v <- y[[iv]]
           if (inherits(v, "list")) {
             v <- unlist(v)
-            if (length(v) == 1) {
+            if (length(v) == 1 & to_list_length != 1) {
               # New col
               y[[iv]] <- paste0(trim(v), ",")
-            } else if ( (length(v) == to_list_length + 1) | (length(v) == 2) ) {
+            } else if (
+              (length(v) == to_list_length + 1) |
+              (length(v) == 2 & to_list_length != 2)
+            ) {
               # New col
               y[[iv]] <- paste0(trim(v[1]), ",", utils::tail(v, -1))
             } else if (length(v) != to_list_length) {
@@ -648,3 +651,240 @@ list_has_curl_handle <- function(x) {
     FALSE
   }
 }
+
+#-------------------------------------------------------------------------------
+# unpack
+unpack <- function(DT) {
+  if (is.null(DT)) {
+    return(NULL)
+  }
+  if (is.list(DT) & !data.table::is.data.table(DT)) {
+    if (length(DT) <= 0) {
+      return(NULL)
+    }
+  }
+  if (nrow(DT) <= 0) {
+    return(NULL)
+  }
+  data.table::setDT(DT)
+  DT[, k := .I]
+  DT <- split(DT, by = "k")
+  lapply(DT, function(DT_) {
+    if (!("children" %in% colnames(DT_))) {
+      DT_[, k := NULL]
+      DT_[]
+    } else {
+      if (is.null(DT_$children[[1]])) {
+        DT_[, k := NULL]
+        DT_[, .SD, .SDcols = setdiff(colnames(DT_), "children")]
+      } else {
+        lapply(DT_$children, unpack)
+      }
+    }
+  })
+}
+
+#-------------------------------------------------------------------------------
+# rbindlist_recursive
+rbindlist_recursive <- function(l) {
+  if (is.null(l)) {
+    return(NULL)
+  }
+  if (data.table::is.data.table(l)) {
+    return(l)
+  }
+  if (
+    sum(sapply(l, is.data.table), na.rm = TRUE) != length(l)
+  ) {
+    l <- lapply(l, rbindlist_recursive)
+  }
+  data.table::rbindlist(l, use.names = TRUE, fill = TRUE)
+}
+
+#-------------------------------------------------------------------------------
+# check_datasets
+check_datasets <- function(l, run = 1) {
+  if (is.null(l)) {
+    return(l)
+  }
+
+  if (run == 0) {
+    return(l)
+  }
+
+  for (i1 in names(l)) {
+    if (is.null(l[[i1]])) {
+      l[[i1]] <- NULL
+    } else {
+      if (length(l[[i1]]) <= 0) {
+        l[[i1]] <- NULL
+      } else {
+        if (nrow(l[[i1]]) <= 0) {
+          l[[i1]] <- NULL
+        }
+      }
+    }
+  }
+
+  check_datasets(l, run = run - 1)
+}
+
+#-------------------------------------------------------------------------------
+# check_dimensions
+check_dimensions <- function(l, run = 1) {
+  if (is.null(l)) {
+    return(l)
+  }
+  
+  if (run == 0) {
+    return(l)
+  }
+
+  for (i1 in names(l)) {
+    if (length(l[[i1]]) <= 0) {
+      l[[i1]] <- NULL
+    } else {
+      for (i2 in names(l[[i1]])) {
+        if (is.null(l[[i1]][[i2]])) {
+          l[[i1]][[i2]] <- NULL
+        } else {
+          if (length(l[[i1]][[i2]]) <= 0) {
+            l[[i1]][[i2]] <- NULL
+          }
+        }
+      }
+    }
+  }
+
+  check_dimensions(l, run = run - 1)
+}
+
+#-------------------------------------------------------------------------------
+# capital_first
+capital_first <- function(x) {
+  if (is.null(x)) {
+    return(x)
+  }
+
+  if (length(x) <= 0) {
+    return(x)
+  }
+
+  paste0(
+    toupper(substr(x, 1, 1)),
+    tolower(substr(x, 2, nchar(x)))
+  )
+}
+
+#-------------------------------------------------------------------------------
+# new_title
+new_title <- function(x) {
+  if (is.null(x)) {
+    return("unknown")
+  }
+
+  if (length(x) <= 0) {
+    return("unknown")
+  }
+
+  if (x != capital_first(x)) {
+    return(capital_first(x))
+  }
+
+  toupper(x)
+}
+
+#-------------------------------------------------------------------------------
+# get_dimensions_values
+get_dimensions_values <- function(DT, ...) {
+  tryCatch({
+    tmpDT <- DT[, .SD, .SDcols = c("provider_code", "dataset_code")]
+    tmpDT <- unique(tmpDT)
+    tmpDT[, k := .I]
+    tmpDT <- lapply(tmpDT$k, function(x) {
+      tmpx <- rdb_dimensions(
+        provider_code = tmpDT[k == x]$provider_code,
+        dataset_code = tmpDT[k == x]$dataset_code,
+        ...
+      )
+      tmpx <- tmpx[[1]][[1]]
+      lapply(tmpx, function(y) {  
+        DT2 <- data.table::copy(y)
+        DT2[, dataset_code := tmpDT[k == x]$dataset_code]
+        data.table::setcolorder(
+          DT2,
+          c("dataset_code", setdiff(colnames(DT2), "dataset_code"))
+        )
+        DT2
+      })
+    })
+    tmpDT <- unlist(tmpDT, recursive = FALSE)  
+    unname(tmpDT)
+  }, error = function(e) {
+    NULL
+  })
+}
+
+#-------------------------------------------------------------------------------
+# get_dimensions_names
+get_dimensions_names <- function(DT = NULL, dimensions_values = NULL, ...) {
+  tryCatch({
+    if (is.null(dimensions_values)) {
+      tmpDT <- get_dimensions_values(DT, ...)
+    } else {
+      tmpDT <- dimensions_values
+    }
+    lapply(tmpDT, function(x) {
+      c(
+        unique(x$dataset_code),
+        utils::tail(colnames(x), -1)
+      )
+    })
+  }, error = function(e) {
+    NULL
+  })
+}
+
+#-------------------------------------------------------------------------------
+# JUST A TRY, DO NOT USE !
+# get_geo_colname2
+get_geo_colname2 <- function(x, y) {
+  u <- try(
+    {
+      nm <- unlist(x)
+      nm <- names(nm)
+      nm <- grep("\\.dimensions_value[s]*_label[s]\\.", nm, value = TRUE)
+      nm <- gsub("\\.dimensions_value[s]*_label[s]", "", nm)
+      nm <- gsub("dataset[s]*\\.*", "", nm)
+      nm <- gsub("\\.[^.]*$", "", nm)
+      nm <- unique(nm)
+
+      nm <- lapply(nm, function(u) {
+        pc <- gsub("/.*", "", u)
+
+        dc <- gsub(paste0(pc, "/"), "", u)
+        dc <- gsub("\\..*", "", dc)
+        
+        code <- gsub(paste0(pc, "/", dc, "\\."), "", u)
+        c(pc, dc, code, new_title(code))
+      })
+      nm <- lapply(nm, `[`, 2:4)
+
+      if (!is.null(y)) {
+        for (i in seq_along(nm)) {
+          for (j in seq_along(y)) {
+            if (nm[[i]][1] == y[[j]][1] & nm[[i]][2] == y[[j]][2]) {
+              nm[[i]][3] <- y[[j]][3]
+            }
+          }
+        }
+      }
+      nm
+    },
+    silent = TRUE
+  )
+  if (inherits(u, "try-error")) {
+    return(y)
+  }
+  u
+}
